/**
 * A class that updates the position of the camera with the help of the arrow Keys and the mouse. This class has a a cameraUpdate()
 * function that sets where the user is looking at and a locationUpdate() function that sets the actual position of the update
 * This class comes from the original code of Ryan D  
 * found February 14, 2016 on (http://www.openprocessing.org/sketch/12557)
 */

class Camera {

  //Camera Variables

  //accelerations of the spaceship
  float acceleration = 0;
  float deceleration = 0;
  float rightAcceleration = 0;
  float leftAcceleration = 0;

  //speedLimit
  float speedLimit = 500;

  //tells if the the spaceship moves or not
  boolean startEngine = false;

  //camera location
  PVector location;

  //scene center or where the user is looking at
  PVector sceneCenter;

  //difference of tx-x and tz-z
  float xComp, zComp;

  //angle of the camera
  float angle;

  //Movement Variables
  boolean moveUp, moveDown, moveLeft, moveRight;

  //Constants

  //height of the camera in the 3D world
  int standHeight = 50;

  //speed of the mouvement of the camera when changing cameraLocation(), a bigger number = slower
  float movementSpeed = 170;  

  //sensitivity to change the locationUpdate(), sensitivy of the mouse when changing where the users is looking at, a bigger numer = slower
  float sensitivity = 35;

  //Center of POV, mouse must be stillBox away from center to move
  int stillBox = 100;       

  //distance from camera to camera target in lookmode
  int cameraDistance = 1000;  

  //Constructor.
  Camera() {

    //camera initialization
    location = new PVector(0, 0, (height/2.0) + 3100 / tan(PI*60.0 / 360.0));

    sceneCenter = new PVector(0, height/2, 0);

    //difference in x and y axis where my eyes is pointing and where the center of the scene is
    xComp = sceneCenter.x - location.x;
    zComp = sceneCenter.z - location.z;

    //sets angle of camera to 0
    angle = 0;

    //Movement Initialization
    moveUp = false;
    moveDown = false;
    moveLeft = false;
    moveRight = false;
  }

  //this function call the locationUpdate function and also call the camera function
  void display() {
    locationUpdate();
    camera(location.x, location.y, location.z, sceneCenter.x, sceneCenter.y, sceneCenter.z, 0, 1, 0);
  }

  //this function updates the position of the camera in the 3D space
  void locationUpdate() {

    //if moveUp is true
    if (moveUp) {

      //sets startEngine tot true
      startEngine = true;

      // accelerates of 4 toward the screen
      acceleration -= 4;

      //updates all x and z parameters of the camera to move toward the screen with an acceleration
      location.z += zComp/(movementSpeed + acceleration);
      sceneCenter.z+= zComp/(movementSpeed + acceleration);
      location.x += xComp/(movementSpeed);
      sceneCenter.x+= xComp/(movementSpeed);

      //else
    } else {

      //if the spaceship is moving
      if (startEngine == true) {

        //if the spaceship goes at a speed smaller than the speedLimit
        if ((movementSpeed + acceleration) < speedLimit) {

          //accelera toward the users, to give a feel of deceleration
          acceleration += 6;

          //updates all x and z parameters of the camera to move forward the screen with a deceleration
          location.z  += zComp/ (movementSpeed + acceleration);
          sceneCenter.z += zComp/ (movementSpeed + acceleration);
          location.x += xComp/(movementSpeed);
          sceneCenter.x += xComp/(movementSpeed);
        }
      }
    }
    //if moveDown is true
    if (moveDown) {

      //decelerates
      deceleration +=  1.5;

      //updates all x and z parameters of the camera to move forward the screen with a deceleration
      location.z  -= zComp/ (movementSpeed + deceleration);
      sceneCenter.z -= zComp/ (movementSpeed + deceleration);
      location.x -= xComp/(movementSpeed);
      sceneCenter.x -= xComp/(movementSpeed);
    }

    //if moveRight is true
    else if (moveRight) {

      //accelerates toward the right
      rightAcceleration += 1.5;

      //updates all x and z parameters of the camera to move to the right of the screen with a acceleration
      location.z += xComp/movementSpeed; 
      sceneCenter.z+= xComp/movementSpeed;
      location.x -= zComp/(movementSpeed + rightAcceleration);
      sceneCenter.x-= zComp/(movementSpeed + rightAcceleration);
    }

    //if moveLeft is true
    else if (moveLeft) {

      //accelerates toward the left
      leftAcceleration += 1.5;

      //updates all x and z parameters of the camera to move to the left of the screen with a acceleration
      location.z -= xComp/movementSpeed; 
      sceneCenter.z-= xComp/movementSpeed;
      location.x += zComp/(movementSpeed + leftAcceleration);
      sceneCenter.x+= zComp/(movementSpeed + leftAcceleration);
    }
  }

  //return the camera location
  PVector getLocation() {
    return location;
  }

  //return the x location of the camera
  float getXLocation() {
    return location.x;
  }

  //return the z location of the camera
  float getZLocation() {
    return location.z;
  }

  //set a newLocation to the camera
  void setLocation(PVector newLocation) {
    location = newLocation;
  }

  //this function changes boolean status when arrowKeys are pressed
  void keyPressed() {
    if (keyPressed) {
      if (key == CODED) {
        if (keyCode == UP) {
          moveUp = true;
        } else if (keyCode == DOWN) {
          moveDown = true;
        } else if (keyCode == LEFT) {
          moveLeft = true;
        } else if (keyCode == RIGHT) {
          moveRight = true;
        }
      }
    }
  }

  //this function changes boolean status when arrowKeys are released
  void keyReleased() {
    if (key == CODED) {
      if (keyCode == UP) {
        moveUp = false;
      } else if (keyCode == DOWN) {
        moveDown = false;
      } else if (keyCode == LEFT) {
        moveLeft = false;
      } else if (keyCode == RIGHT) {
        moveRight = false;
      }
    }
  }
}