/**
 * A abstract class describing a physical celestial object (Pshape) in a 3D space. The celestial object has a location (x,y, z), a mass, 
 * a orignal mass, a texture image. This class is not displayed in the main tab, it is only used as a Parent class
 */
abstract class CelestialObject {

  //location
  PVector location;

  //mass
  float mass;

  //orignal mass
  float originalMass;

  //a shape for the celestial object
  PShape shape;

  //PImage for the texture
  PImage img;

  //Constructor. CelestialObject can be created with a locaiton, mass and texturefile name
  CelestialObject(PVector location_, float mass_, String textureFileName) {

    //Assigns values to data and new PVector to data
    location = location_;
    mass = mass_;
    originalMass = mass;

    //creates a sphere
    shape = createShape(SPHERE, mass);

    //loads textures
    img = loadImage(textureFileName);
  }

  //this function displays the celestial object
  void display() {

    //scales the object according to its original mass
    shape.scale(mass/originalMass);

    //sets texture to the shape
    shape.setTexture(img);

    //draw the shpere
    shape(shape);
  }

  //updates the position of the celestial object. Need to be overriden
  void update() {
  }

  //returns location of the celestial object
  PVector getLocation() {
    return location;
  }

  //returns mass of the celestial object
  float getMass() {
    return mass;
  }

  //sets a new mass
  void setMass(float newMass) {
    mass = newMass;
  }

  //----All the followed functions need to be overriden in the Planet class--------///

  //sets a new speed orbit
  void setSpeedOrbit(float newSpeedOrbit) {
  }

  //sets a new orbit
  void setRadius(float newRadius) {
  }

  //sets a new rotation
  void setSpeedRotation(float newSpeedRotation) {
  }

  //sets a new gravitaiton field
  void setGravitationalField(float newGravitationalField) {
  }

  //sets a new planet Name
  void setPlanetName(String newPlanetName) {
  }

  //sets a new state to the boolean displayPlanetNames
  void setStateDisplayPlanetNames(boolean newState) {
  }

  //does action when key is pressed
  void keyPressed() {
  }
}