/**
 * This class is the view and controller of the spaceship. All the controllers of the UI are contained in this class. 
 * The class class displays the spaceship cockpit and the spaceship control panel
 */
class ControlPanel {

  //initializes the controlP5 library
  ControlP5 cp5;

  //a reference to the model universe
  Universe universe;

  //instantiates the cockpit image
  PImage cockpit;

  //x and y offset of the orbit knob
  final float  X_OFFSET_ORBIT_KNOB = 180;
  final float Y_OFFSET_ORBIT_KNOB = 230; 

  //x and y offset of the rotation knob
  final float X_OFFSET_ROTATION_KNOB = 80;
  final float Y_OFFSET_ROTATION_KNOB = 230;

  //x and y offset of the mass slider
  final float X_OFFSET_MASS_SLIDER = 180;
  final float Y_OFFSET_MASS_SLIDER = 130;

  //x and y offset of the radisu slider
  final float X_OFFSET_RADIUS_SLIDER = 180;
  final float Y_OFFSET_RADIUS_SLIDER = 90;

  //x and y offset of the gravitation slider
  final float X_OFFSET_GRAVITATION_SLIDER = 30;
  final float Y_OFFSET_GRAVITATION_SLIDER = 180;

  //x and y offset of the toggle
  final float X_OFFSET_TOGGLE = 80;
  final float Y_OFFSET_TOGGLE = 230;

  //x and y offset of the textField
  final float X_OFFSET_TEXTFIELD = 20;
  final float Y_OFFSET_TEXTFIELD = 230;

  //size of the knobs
  final float KNOB_SIZE = 33;

  //number of tickmarks aroun a knob
  final int NUM_TICKMARKS = 10;

  //number of tickmarks on a slider
  final int NUM_SLIDER_TICKMARKS = 3;

  //length of the tickmarks
  final int KNOB_TICKMARKS_LENGTH  = 4;

  //width and height of the vertical slider
  final int VERTICAL_SLIDER_WIDTH = 25;
  final int VERTICAL_SLIDER_HEIGHT = 100;

  ////width and height of the horizontal sliders
  final int HORIZONTAL_SLIDER_WIDTH = 150;
  final int HORIZONTAL_SLIDER_HEIGHT = 15;

  //width and height of the buttons
  final int BUTTON_WIDTH = 100;
  final int BUTTON_HEIGHT = 40;

  //width and height of the textfield
  final int TEXTFIELD_WIDTH = 200;
  final int TEXTFIELD_HEIGHT = 20;

  //width and height of the Toggle
  final int TOGGLE_WIDHT = 70;
  final int TOGGLE_HEIGHT = 30;

  //range of the planets speed orbit around the sun
  final float MIN_ORBIT = 70000;
  final float MAX_ORBIT = 3000;

  //range of the planets rotation around themselves
  final float MIN_ROTATION = 0;
  final float MAX_ROTATION = 40;

  //range of the radius/distance between planets and sun
  final float MIN_RADIUS = 800;
  final float MAX_RADIUS = 4500;

  //range of the gravitational field of the planets
  final float MIN_GRAVITATION = 100;
  final float MAX_GRAVITATION = 1000;

  //foreground, background and active color of the controllers
  final color FOREGROUND_COLOR = color(35, 232, 174);
  final color BACKGROUND_COLOR = color(26, 78, 77);
  final color ACTIVE_COLOR = color(124, 255, 217);

  //Starting values(speed orbit, mass, radius, gravitationField) of the blue planet
  final float BLUE_SPEED_ORBIT = 20000;
  final float BLUE_MASS = 60;
  final float BLUE_RADIUS = 800;
  final float BLUE_GRAVITATION = 200;

  //Starting values(speed orbit, mass, radius, gravitationField) of the pink planet
  final float PINK_SPEED_ORBIT = 28000;
  final float PINK_MASS = 100;
  final float PINK_RADIUS = 1300;
  final float PINK_GRAVITATION = 200;

  //Starting values(speed orbit, mass, radius, gravitationField) of the green planet
  final float GREEN_SPEED_ORBIT = 45000;
  final  float GREEN_MASS = 75;
  final float GREEN_RADIUS = 2000;
  final float GREEN_GRAVITATION = 100;

  //Starting values(speed orbit, mass, radius, gravitationField) of the purple planet
  final float PURPLE_SPEED_ORBIT = 21000;
  final float PURPLE_MASS = 185;
  final float PURPLE_RADIUS = 3000;
  final float PURPLE_GRAVITATION = 300;

  //Starting values(speed orbit, mass, radius, gravitationField) of the red planet
  final float RED_SPEED_ORBIT = 30000;
  final float RED_MASS = 40;
  final float RED_RADIUS = 3600;
  final float RED_GRAVITATION = 100;

  //Starting values(speed orbit, mass, radius, gravitationField) of the orange planet
  final float ORANGE_SPEED_ORBIT = 60000;
  final float ORANGE_MASS = 90;
  final float ORANGE_RADIUS = 4000;
  final float ORANGE_GRAVITATION = 200;

  //Constructor with a PApplet and universe parameter
  ControlPanel(PApplet thePApplet, Universe universe) {
    this.universe = universe;

    //instantiate the user interface controlP5
    cp5 = new ControlP5(thePApplet);

    //load the cockpit image
    //Design by Maelle Morin
    //Inspired by the spaceship in the game Evochron Mercenary
    cockpit = loadImage("spaceship.png");
  }

  //function to add all the controllers of the UI
  void addControlPanel() {
    addTabs();
    addControllersPlanets();
    addControllerSpaceship();
  }

  //adds the tabs
  void addTabs() {

    //changes the name of the default tab
    cp5.getTab("default")
      .setLabel("Menu")
      ;

    //creates a tab for the general spaceship
    cp5.addTab("Spaceship");

    //creates 6 tabs for each planets
    cp5.addTab("Blue Planet");
    cp5.addTab("Pink Planet");
    cp5.addTab("Green Planet");
    cp5.addTab("Purple Planet");
    cp5.addTab("Red Planet");
    cp5.addTab("Orange Planet");
  }

  //adds all the planets controllers
  void addControllersPlanets() {
    addControllerBluePlanet();
    addControllerPinkPlanet();
    addControllerGreenPlanet();
    addControllerPurplePlanet();
    addControllerRedPlanet();
    addControllerOrangePlanet();
  }

  //adds all the general spaceship controllers
  void addControllerSpaceship() {
    addButtons();
    addToggle();
  }

  //adds a toggle
  void addToggle() {
    cp5.addToggle("displayNames")

      //sets position, size, value, mode of the toggle
      .setPosition(width/2 + X_OFFSET_TOGGLE, height - Y_OFFSET_TOGGLE)
      .setSize(TOGGLE_WIDHT, TOGGLE_HEIGHT)
      .setValue(false)
      .setMode(ControlP5.SWITCH)

      //sets the foreground, background and active color
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //Places the toogle in the spaceship tab
    cp5.getController("displayNames").moveTo("Spaceship");
  }

  //add the buttons
  void addButtons() {

    //adds a button to add blue particles
    cp5.addButton("blue_Particles")

      //sets value, position, size, foreground, background and active color of the button
      .setValue(1)
      .setPosition(width/2 - 190, height - 230)
      .setSize(BUTTON_WIDTH, BUTTON_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a button to add pink particles
    cp5.addButton("pink_Particles")

      //sets value, position, size, foreground, background and active color of the button
      .setValue(2)
      .setPosition(width/2 - 50, height - 230)
      .setSize(BUTTON_WIDTH, BUTTON_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a button to add green particles
    cp5.addButton("green_Particles")

      //sets value, position, size, foreground, background and active color of the button
      .setValue(2)
      .setValue(3)
      .setPosition(width/2 - 190, height - 170)
      .setSize(BUTTON_WIDTH, BUTTON_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a button to add purple particles
    cp5.addButton("purple_Particles")

      //sets value, position, size, foreground, background and active color of the button
      .setValue(4)
      .setPosition(width/2 - 50, height - 170)
      .setSize(BUTTON_WIDTH, BUTTON_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a button to add red particles
    cp5.addButton("red_Particles")

      //sets value, position, size, foreground, background and active color of the button
      .setValue(5)
      .setPosition(width/2 - 190, height - 110)
      .setSize(BUTTON_WIDTH, BUTTON_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a button to add orange particles
    cp5.addButton("orange_Particles")

      //sets value, position, size, foreground, background and active color of the button
      .setValue(6)
      .setPosition(width/2 - 50, height - 110)
      .setSize(BUTTON_WIDTH, BUTTON_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the buttons in the spaceship tab
    cp5.getController("blue_Particles").moveTo("Spaceship");
    cp5.getController("pink_Particles").moveTo("Spaceship");
    cp5.getController("green_Particles").moveTo("Spaceship");
    cp5.getController("purple_Particles").moveTo("Spaceship");
    cp5.getController("red_Particles").moveTo("Spaceship");
    cp5.getController("orange_Particles").moveTo("Spaceship");
  }

  //adds controllers to change parameters of the blue planet
  void addControllerBluePlanet() {

    //adds a knob to change the speed orbit
    cp5.addKnob("speedOrbit1")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ORBIT, MAX_ORBIT)
      .setValue(BLUE_SPEED_ORBIT)
      .setPosition(width/2 - X_OFFSET_ORBIT_KNOB, height - Y_OFFSET_ORBIT_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a knob to change the rotation 
    cp5.addKnob("speedRotation1")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ROTATION, MAX_ROTATION)
      .shuffle()
      .setPosition(width/2 - X_OFFSET_ROTATION_KNOB, height - Y_OFFSET_ROTATION_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the mass
    cp5.addSlider("mass1")

      //sets the range, value, size, position,  number  of tickmarks, 
      //foreground, background and active color of the slider
      .setRange(BLUE_MASS - 1, BLUE_MASS + 1)
      .setValue(BLUE_MASS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_MASS_SLIDER, height - Y_OFFSET_MASS_SLIDER)
      .setNumberOfTickMarks(NUM_SLIDER_TICKMARKS)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the radius
    cp5.addSlider("radius1")

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_RADIUS, MAX_RADIUS)
      .setValue(BLUE_RADIUS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_RADIUS_SLIDER, height - Y_OFFSET_RADIUS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the gravitational field
    cp5.addSlider("gravitationalField1") 

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_GRAVITATION, MAX_GRAVITATION)
      .setValue(BLUE_GRAVITATION)
      .setSize(VERTICAL_SLIDER_WIDTH, VERTICAL_SLIDER_HEIGHT)
      .setPosition(width/2 + X_OFFSET_GRAVITATION_SLIDER, height - Y_OFFSET_GRAVITATION_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a textfield to change the name 
    cp5.addTextfield("name1")

      //sets position, size
      //foreground, background and active color of the textfield
      .setPosition(width/2 + X_OFFSET_TEXTFIELD, height - Y_OFFSET_TEXTFIELD)
      .setSize(TEXTFIELD_WIDTH, TEXTFIELD_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the controller in the blue planet tab
    cp5.getController("speedOrbit1").moveTo("Blue Planet");
    cp5.getController("speedRotation1").moveTo("Blue Planet");
    cp5.getController("mass1").moveTo("Blue Planet");
    cp5.getController("radius1").moveTo("Blue Planet");
    cp5.getController("gravitationalField1").moveTo("Blue Planet");
    cp5.getController("name1").moveTo("Blue Planet");
  }

  //adds controllers to change parameters of the pink planet
  void addControllerPinkPlanet() {

    //adds a knob to change the speed orbit
    cp5.addKnob("speedOrbit2")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ORBIT, MAX_ORBIT)
      .setValue(PINK_SPEED_ORBIT)
      .setPosition(width/2 - X_OFFSET_ORBIT_KNOB, height - Y_OFFSET_ORBIT_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a knob to change the rotation 
    cp5.addKnob("speedRotation2")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ROTATION, MAX_ROTATION)
      .shuffle()
      .setPosition(width/2 - X_OFFSET_ROTATION_KNOB, height - Y_OFFSET_ROTATION_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the mass
    cp5.addSlider("mass2")

      //sets the range, value, size, position,  number  of tickmarks, 
      //foreground, background and active color of the slider
      .setRange(PINK_MASS - 1, PINK_MASS + 1)
      .setValue(PINK_MASS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_MASS_SLIDER, height - Y_OFFSET_MASS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      .setNumberOfTickMarks(NUM_SLIDER_TICKMARKS)
      ;

    //adds a slider to change the radius
    cp5.addSlider("radius2")

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_RADIUS, MAX_RADIUS)
      .setValue(PINK_RADIUS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_RADIUS_SLIDER, height - Y_OFFSET_RADIUS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the gravitational field
    cp5.addSlider("gravitationalField2") 

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_GRAVITATION, MAX_GRAVITATION)
      .setValue(PINK_GRAVITATION)
      .setSize(VERTICAL_SLIDER_WIDTH, VERTICAL_SLIDER_HEIGHT)
      .setPosition(width/2 + X_OFFSET_GRAVITATION_SLIDER, height - Y_OFFSET_GRAVITATION_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a textfield to change the name 
    cp5.addTextfield("name2")

      //sets position, size
      //foreground, background and active color of the textfield
      .setPosition(width/2 + X_OFFSET_TEXTFIELD, height - Y_OFFSET_TEXTFIELD)
      .setSize(TEXTFIELD_WIDTH, TEXTFIELD_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the controller in the pink planet tab
    cp5.getController("speedOrbit2").moveTo("Pink Planet");
    cp5.getController("speedRotation2").moveTo("Pink Planet");
    cp5.getController("mass2").moveTo("Pink Planet");
    cp5.getController("radius2").moveTo("Pink Planet");
    cp5.getController("gravitationalField2").moveTo("Pink Planet");
    cp5.getController("name2").moveTo("Pink Planet");
  }


  //adds controllers to change parameters of the green planet
  void addControllerGreenPlanet() {

    //adds a knob to change the speed orbit
    cp5.addKnob("speedOrbit3")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ORBIT, MAX_ORBIT)
      .setValue(GREEN_SPEED_ORBIT)
      .setPosition(width/2 - X_OFFSET_ORBIT_KNOB, height - Y_OFFSET_ORBIT_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a knob to change the rotation 
    cp5.addKnob("speedRotation3")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob 
      .setRange(MIN_ROTATION, MAX_ROTATION)
      .shuffle()
      .setPosition(width/2 - X_OFFSET_ROTATION_KNOB, height - Y_OFFSET_ROTATION_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the mass
    cp5.addSlider("mass3")

      //sets the range, value, size, position,  number  of tickmarks, 
      //foreground, background and active color of the slider
      .setRange(GREEN_MASS - 1, GREEN_MASS + 1)
      .setValue(GREEN_MASS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_MASS_SLIDER, height - Y_OFFSET_MASS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      .setNumberOfTickMarks(NUM_SLIDER_TICKMARKS)
      ;

    //adds a slider to change the radius
    cp5.addSlider("radius3")

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_RADIUS, MAX_RADIUS)
      .setValue(GREEN_RADIUS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_RADIUS_SLIDER, height - Y_OFFSET_RADIUS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the gravitational field
    cp5.addSlider("gravitationalField3") 

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_GRAVITATION, MAX_GRAVITATION)
      .setValue(GREEN_GRAVITATION)
      .setSize(VERTICAL_SLIDER_WIDTH, VERTICAL_SLIDER_HEIGHT)
      .setPosition(width/2 + X_OFFSET_GRAVITATION_SLIDER, height - Y_OFFSET_GRAVITATION_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a textfield to change the name 
    cp5.addTextfield("name3")

      //sets position, size
      //foreground, background and active color of the textfield
      .setPosition(width/2 + X_OFFSET_TEXTFIELD, height - Y_OFFSET_TEXTFIELD)
      .setSize(TEXTFIELD_WIDTH, TEXTFIELD_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the controller in the green planet tab
    cp5.getController("speedOrbit3").moveTo("Green Planet");
    cp5.getController("speedRotation3").moveTo("Green Planet");
    cp5.getController("mass3").moveTo("Green Planet");
    cp5.getController("radius3").moveTo("Green Planet");
    cp5.getController("gravitationalField3").moveTo("Green Planet");
    cp5.getController("name3").moveTo("Green Planet");
  }

  //adds controllers to change parameters of the purple planet
  void addControllerPurplePlanet() {

    //adds a knob to change the speed orbit
    cp5.addKnob("speedOrbit4")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ORBIT, MAX_ORBIT)
      .setValue(PURPLE_SPEED_ORBIT)
      .setPosition(width/2 - X_OFFSET_ORBIT_KNOB, height - Y_OFFSET_ORBIT_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a knob to change the rotation 
    cp5.addKnob("speedRotation4")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ROTATION, MAX_ROTATION)
      .shuffle()
      .setPosition(width/2 - X_OFFSET_ROTATION_KNOB, height - Y_OFFSET_ROTATION_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the mass
    cp5.addSlider("mass4")

      //sets the range, value, size, position,  number  of tickmarks, 
      //foreground, background and active color of the slider
      .setRange(PURPLE_MASS - 1, PURPLE_MASS + 1)
      .setValue(PURPLE_MASS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_MASS_SLIDER, height - Y_OFFSET_MASS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      .setNumberOfTickMarks(NUM_SLIDER_TICKMARKS)
      ;

    //adds a slider to change the radius
    cp5.addSlider("radius4")

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_RADIUS, MAX_RADIUS)
      .setValue(PURPLE_RADIUS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_RADIUS_SLIDER, height - Y_OFFSET_RADIUS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the gravitational field
    cp5.addSlider("gravitationalField4") 

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_GRAVITATION, MAX_GRAVITATION)
      .setValue(PURPLE_GRAVITATION)
      .setSize(VERTICAL_SLIDER_WIDTH, VERTICAL_SLIDER_HEIGHT)
      .setPosition(width/2 + X_OFFSET_GRAVITATION_SLIDER, height - Y_OFFSET_GRAVITATION_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a textfield to change the name
    cp5.addTextfield("name4")

      //sets position, size
      //foreground, background and active color of the textfield
      .setPosition(width/2 + X_OFFSET_TEXTFIELD, height - Y_OFFSET_TEXTFIELD)
      .setSize(TEXTFIELD_WIDTH, TEXTFIELD_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the controller in the purple planet tab
    cp5.getController("speedOrbit4").moveTo("Purple Planet");
    cp5.getController("speedRotation4").moveTo("Purple Planet");
    cp5.getController("mass4").moveTo("Purple Planet");
    cp5.getController("radius4").moveTo("Purple Planet");
    cp5.getController("gravitationalField4").moveTo("Purple Planet");
    cp5.getController("name4").moveTo("Purple Planet");
  }

  //adds controllers to change parameters of the red planet
  void addControllerRedPlanet() {

    //adds a knob to change the speed orbit
    cp5.addKnob("speedOrbit5")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ORBIT, MAX_ORBIT)
      .setValue(RED_SPEED_ORBIT)
      .setPosition(width/2 - X_OFFSET_ORBIT_KNOB, height - Y_OFFSET_ORBIT_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a knob to change the rotation 
    cp5.addKnob("speedRotation5")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ROTATION, MAX_ROTATION)
      .shuffle()
      .setPosition(width/2 - X_OFFSET_ROTATION_KNOB, height - Y_OFFSET_ROTATION_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the mass
    cp5.addSlider("mass5")

      //sets the range, value, size, position,  number  of tickmarks, 
      //foreground, background and active color of the slider
      .setRange(RED_MASS - 1, RED_MASS + 1)
      .setValue(RED_MASS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_MASS_SLIDER, height - Y_OFFSET_MASS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      .setNumberOfTickMarks(NUM_SLIDER_TICKMARKS)
      ;

    //adds a slider to change the radius
    cp5.addSlider("radius5")

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_RADIUS, MAX_RADIUS)
      .setValue(RED_RADIUS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_RADIUS_SLIDER, height - Y_OFFSET_RADIUS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the gravitational field
    cp5.addSlider("gravitationalField5") 

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_GRAVITATION, MAX_GRAVITATION)
      .setValue(RED_GRAVITATION)
      .setSize(VERTICAL_SLIDER_WIDTH, VERTICAL_SLIDER_HEIGHT)
      .setPosition(width/2 + X_OFFSET_GRAVITATION_SLIDER, height - Y_OFFSET_GRAVITATION_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a textfield to change the name 
    cp5.addTextfield("name5")

      //sets position, size
      //foreground, background and active color of the textfield
      .setPosition(width/2 + X_OFFSET_TEXTFIELD, height - Y_OFFSET_TEXTFIELD)
      .setSize(TEXTFIELD_WIDTH, TEXTFIELD_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the controller in the red planet tab
    cp5.getController("speedOrbit5").moveTo("Red Planet");
    cp5.getController("speedRotation5").moveTo("Red Planet");
    cp5.getController("mass5").moveTo("Red Planet");
    cp5.getController("radius5").moveTo("Red Planet");
    cp5.getController("gravitationalField5").moveTo("Red Planet");
    cp5.getController("name5").moveTo("Red Planet");
  }

  //adds controllers to change parameters of the blue planet
  void addControllerOrangePlanet() {

    //adds a knob to change the speed orbit
    cp5.addKnob("speedOrbit6")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ORBIT, MAX_ORBIT)
      .setValue(ORANGE_SPEED_ORBIT)
      .setPosition(width/2 - X_OFFSET_ORBIT_KNOB, height - Y_OFFSET_ORBIT_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a knob to change the rotation 
    cp5.addKnob("speedRotation6")

      //sets the range, value, position, radius, number and length of tickmarks, 
      //drag direction, foreground, background and active color of the knob
      .setRange(MIN_ROTATION, MAX_ROTATION)
      .shuffle()
      .setPosition(width/2 - X_OFFSET_ROTATION_KNOB, height - Y_OFFSET_ROTATION_KNOB)
      .setRadius(KNOB_SIZE)
      .setNumberOfTickMarks(NUM_TICKMARKS)
      .setTickMarkLength(KNOB_TICKMARKS_LENGTH)
      .setDragDirection(Knob.HORIZONTAL)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the mass
    cp5.addSlider("mass6")

      //sets the range, value, size, position,  number  of tickmarks, 
      //foreground, background and active color of the slider
      .setRange(ORANGE_MASS - 1, ORANGE_MASS + 1)
      .setValue(ORANGE_MASS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_MASS_SLIDER, height - Y_OFFSET_MASS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      .setNumberOfTickMarks(NUM_SLIDER_TICKMARKS)
      ;

    //adds a slider to change the radius
    cp5.addSlider("radius6")

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_RADIUS, MAX_RADIUS)
      .setValue(ORANGE_RADIUS)
      .setSize(HORIZONTAL_SLIDER_WIDTH, HORIZONTAL_SLIDER_HEIGHT)
      .setPosition(width/2 - X_OFFSET_RADIUS_SLIDER, height - Y_OFFSET_RADIUS_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a slider to change the gravitational field
    cp5.addSlider("gravitationalField6") 

      //sets the range, value, size, position, 
      //foreground, background and active color of the slider
      .setRange(MIN_GRAVITATION, MAX_GRAVITATION)
      .setValue(ORANGE_GRAVITATION)
      .setSize(VERTICAL_SLIDER_WIDTH, VERTICAL_SLIDER_HEIGHT)
      .setPosition(width/2 + X_OFFSET_GRAVITATION_SLIDER, height - Y_OFFSET_GRAVITATION_SLIDER)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //adds a textfield to change the name 
    cp5.addTextfield("name6")

      //sets position, size
      //foreground, background and active color of the textfield
      .setPosition(width/2 + X_OFFSET_TEXTFIELD, height - Y_OFFSET_TEXTFIELD)
      .setSize(TEXTFIELD_WIDTH, TEXTFIELD_HEIGHT)
      .setColorForeground(FOREGROUND_COLOR)
      .setColorBackground(BACKGROUND_COLOR)
      .setColorActive(ACTIVE_COLOR)
      ;

    //places all the controller in the blue planet tab
    cp5.getController("speedOrbit6").moveTo("Orange Planet");
    cp5.getController("speedRotation6").moveTo("Orange Planet");
    cp5.getController("mass6").moveTo("Orange Planet");
    cp5.getController("radius6").moveTo("Orange Planet");
    cp5.getController("gravitationalField6").moveTo("Orange Planet");
    cp5.getController("name6").moveTo("Orange Planet");
  }

  //this function displays the cockpit 
  void displayCockpit() {
    image(cockpit, 0, 0, width, height);
  }

  //this function displays the position of the cockpit
  void spaceshipPosition(float cameraXLocation, float cameraZLocation) {

    //fills the text with white
    fill(255);

    //sets the font and size of the text
    textFont(font);
    textSize(12);

    //displays the text
    text("POSITION", width/2 + 100, height - 150);
    text("X: " + cameraXLocation, width/2 + 100, height - 130);
    text("Z: " + cameraZLocation, width/2 + 100, height - 110);

    //sets no fill and a stroke to blue
    noFill();
    stroke(58, 167, 242);

    //displays a rect around the text
    rectMode(CENTER);
    rect(width/2 + 150, height - 135, 120, 70);
  }

  //this function calls the keyPressed function of the different planets
  void keyPressed() {

    if (planet == 1) {

      //calls the keyPressed function of the planet 1 (blue planet)
      universe.celestialObjects.get(1).keyPressed();
    } else if (planet == 2) {

      //calls the keyPressed function of the planet 2 (pink planet)
      universe.celestialObjects.get(2).keyPressed();
    } else if (planet == 3) {

      //calls the keyPressed function of the planet 3 (green planet)
      universe.celestialObjects.get(3).keyPressed();
    } else if (planet == 4) {

      //calls the keyPressed function of the planet 4 (purple planet)
      universe.celestialObjects.get(4).keyPressed();
    } else if (planet == 5) {

      //calls the keyPressed function of the planet 5 (red planet)
      universe.celestialObjects.get(5).keyPressed();
    } else if (planet == 6) {

      //calls the keyPressed function of the planet 6 (orange planet)
      universe.celestialObjects.get(6).keyPressed();
    }
  }
}

//***** End of the ControlPanel class *****//


//********** These functions need to be outide an inner classs, even if it should be in the ControlPanel **************//
//********** but otherwise the ControlP5 doesn't work **************//

//---------- Functions to set new parameters for the blue planet ----------------///

//sets a new speed orbit
void speedOrbit1(int newSpeedOrbit1) {
  universe.celestialObjects.get(1).setSpeedOrbit(newSpeedOrbit1);
}

//sets a new speed rotation
void speedRotation1(int newSpeedRotation1) {
  universe.celestialObjects.get(1).setSpeedRotation(newSpeedRotation1);
}

//sets new mass
void mass1 (int newMass1) {
  universe.celestialObjects.get(1).setMass(newMass1);
}

//sets new mass
void radius1 (int newRadius1) {
  universe.celestialObjects.get(1).setRadius(newRadius1);
}

//sets new gravitational field
void gravitationalField1(int newGravitationalField1) {
  universe.celestialObjects.get(1).setGravitationalField(newGravitationalField1);
}

//sets new name
void name1(String newName1) {
  universe.celestialObjects.get(1).setPlanetName(newName1);
}

//---------------------------------------------------------------------------//

//---------- Functions to set new parameters for the pink planet ----------------///

//sets a new speed orbit
void speedOrbit2(int newSpeedOrbit2) {
  universe.celestialObjects.get(2).setSpeedOrbit(newSpeedOrbit2);
}

//sets a new speed rotation
void speedRotation2(int newSpeedRotation2) {
  universe.celestialObjects.get(2).setSpeedRotation(newSpeedRotation2);
}

//sets new mass
void mass2 (int newMass2) {
  universe.celestialObjects.get(2).setMass(newMass2);
}

//sets new radius
void radius2 (int newRadius2) {
  universe.celestialObjects.get(2).setRadius(newRadius2);
}

//sets new gravitational field
void gravitationalField2(int newGravitationalField2) {
  universe.celestialObjects.get(2).setGravitationalField(newGravitationalField2);
}

//sets new name
void name2(String newName2) {
  universe.celestialObjects.get(2).setPlanetName(newName2);
}

//---------------------------------------------------------------------------//

//---------- Functions to set new parameters for the green planet ----------------///

//sets a new speed orbit
void speedOrbit3(int newSpeedOrbit3) {
  universe.celestialObjects.get(3).setSpeedOrbit(newSpeedOrbit3);
}

//sets a new speed rotation
void speedRotation3(int newSpeedRotation3) {
  universe.celestialObjects.get(3).setSpeedRotation(newSpeedRotation3);
}

//sets new mass
void mass3 (int newMass3) {
  universe.celestialObjects.get(3).setMass(newMass3);
}

//sets new radius
void radius3 (int newRadius3) {
  universe.celestialObjects.get(3).setRadius(newRadius3);
}

//sets new gravitational field
void gravitationalField3(int newGravitationalField3) {
  universe.celestialObjects.get(3).setGravitationalField(newGravitationalField3);
}

//sets new name
void name3(String newName3) {
  universe.celestialObjects.get(3).setPlanetName(newName3);
}

//---------------------------------------------------------------------------//

//---------- Functions to set new parameters for the purple planet ----------------///

//sets a new speed orbit
void speedOrbit4(int newSpeedOrbit4) {
  universe.celestialObjects.get(4).setSpeedOrbit(newSpeedOrbit4);
}

//sets a new speed rotation
void speedRotation4(int newSpeedRotation4) {
  universe.celestialObjects.get(4).setSpeedRotation(newSpeedRotation4);
}

//sets new mass
void mass4 (int newMass4) {
  universe.celestialObjects.get(4).setMass(newMass4);
}

//sets new radius
void radius4 (int newRadius4) {
  universe.celestialObjects.get(4).setRadius(newRadius4);
}

//sets new gravitational field
void gravitationalField4(int newGravitationalField4) {
  universe.celestialObjects.get(4).setGravitationalField(newGravitationalField4);
}

//sets new name
void name4(String newName4) {
  universe.celestialObjects.get(4).setPlanetName(newName4);
}

//---------------------------------------------------------------------------//

//---------- Functions to set new parameters for the red planet ----------------//

//sets a new speed orbit
void speedOrbit5(int newSpeedOrbit5) {
  universe.celestialObjects.get(5).setSpeedOrbit(newSpeedOrbit5);
}

//sets a new speed rotation
void speedRotation5(int newSpeedRotation5) {
  universe.celestialObjects.get(5).setSpeedRotation(newSpeedRotation5);
}

//sets new mass
void mass5 (int newMass5) {
  universe.celestialObjects.get(5).setMass(newMass5);
}

//sets new radius
void radius5 (int newRadius5) {
  universe.celestialObjects.get(5).setRadius(newRadius5);
}

//sets new gravitational field
void gravitationalField5(int newGravitationalField5) {
  universe.celestialObjects.get(5).setGravitationalField(newGravitationalField5);
}

//sets new name
void name5(String newName5) {
  universe.celestialObjects.get(5).setPlanetName(newName5);
}

//---------------------------------------------------------------------------//

//---------- Functions to set new parameters for the orange planet ----------------//

//sets a new speed orbit
void speedOrbit6(int newSpeedOrbit6) {
  universe.celestialObjects.get(6).setSpeedOrbit(newSpeedOrbit6);
}

//sets a new speed rotation
void speedRotation6(int newSpeedRotation6) {
  universe.celestialObjects.get(6).setSpeedRotation(newSpeedRotation6);
}

//sets new mass
void mass6 (int newMass6) {
  universe.celestialObjects.get(6).setMass(newMass6);
}

//sets new radius
void radius6 (int newRadius6) {
  universe.celestialObjects.get(6).setRadius(newRadius6);
}

//sets new gravitational field
void gravitationalField6(int newGravitationalField6) {
  universe.celestialObjects.get(6).setGravitationalField(newGravitationalField6);
}

//sets new name
void name6(String newName6) {
  universe.celestialObjects.get(6).setPlanetName(newName6);
}

//---------------------------------------------------------------------------//


//tells to which planet the user add new particles
int planet = 0;

//sets a new value to planet
void blue_Particles(int blue) {
  planet = blue;
}

//sets a new value to planet
void pink_Particles(int pink) {
  planet = pink;
}

//sets a new value to planet
void green_Particles(int green) {
  planet = green;
}

//sets a new value to planet
void purple_Particles(int purple) {
  planet = purple;
}

//sets a new value to planet
void red_Particles(int red) {
  planet = red;
}

//sets a new value to planet
void orange_Particles(int orange) {
  planet = orange;
}

//sets a new state to displayNames 
void displayNames(boolean displayNames_) {

  //for each celestial objects
  for (int i = 0; i < universe.celestialObjects.size(); i++) {

    //changes the state of the displayPlanetName boolean
    CelestialObject co = universe.celestialObjects.get(i);
    co.setStateDisplayPlanetNames(displayNames_);
  }
}