/**
 * A class to describe a group of Particles. An ArrayList is used to manage the list of Particles.
 * The particle system has a origin point and the gravitation field of the planet
 */

class ParticleSystem {

  // An arraylist for all the particles.
  ArrayList<Particle> particles;

  // An origin point for where particles are birthed.
  PVector origin;

  //gravitation field of the planet
  float gravitationalField;

  //constructor with the origin location
  ParticleSystem(PVector origin_) {

    // Initializes the arraylist.
    particles = new ArrayList<Particle>();

    // Stores the origin point
    origin = origin_.get();

    //creates 50 particles
    for (int i=0; i< 50; i++) {
      particles.add(createParticle());
    }
  }

  // This method is called at every step. Applies the forces on particles and updates them.
  void run(PVector newOrigin, float planetMass, float gravitationalField) {

    //pass the origin parameter 
    origin = newOrigin;

    //calls the applySeeking functions
    applySeeking(planetMass, gravitationalField);

    //for every particles in the arraylist
    for (int i = particles.size()-1; i >= 0; i--) {

      //calls the run function
      Particle p = particles.get(i);
      p.run();
    }
  }

  //adds aparticle
  void addParticle() {
    particles.add(createParticle());
  }

  // Creates and returns a new particle at a random position.
  //substract the origin and width/2 and height/2 to "cancel" the previous translate
  Particle createParticle() {
    return new Particle(random(1, 6), random(width) - (origin.x + width/2), random(height) - (origin.y + height/2), 0);
  }

  // Applies seeking behavior for all the particles
  void applySeeking(float mass, float gravitationalField) {

    //for each particles
    for (Particle p : particles) {

      //distance between the location of the particle and the planet
      PVector distance = PVector.sub(p.location, new PVector(0, 0, 0));

      //if the distance is greater than the planet gravitational field
      if (distance.mag() > gravitationalField) {

        //seeks toward the planet
        p.seek(new PVector(0, 0, 0));
      } else {      

        //go away of the planet
        p.goAway(new PVector(0, 0, 0), mass);
      }
    }
  }
}