/** 
 * A universe class that contains all the object of my universe (the celestial objects, particle systems, background) and a camera.
 * The camera allows the users to explore the universe.
 */

class Universe {

  //creates a PImage as the background
  PImage background;

  //arrayList of celestial object
  ArrayList<CelestialObject> celestialObjects;

  //an array of colors
  color[] colors;

  //delcares a camera
  Camera myCamera;

  //Constructor.
  Universe() {

    //instatiates the arrayList celestialObject
    celestialObjects = new ArrayList<CelestialObject>();

    //instantiates the colors array
    colors = new color[6];

    //instantiates the camera
    myCamera = new Camera();

    //loads background image
    //Image by Karissa(ex-astris1701)
    //artist authorizes the use of the image
    //http://fav.me/d48gc0f
    background = loadImage("background.jpg");

    //for each index in the array colors, assign a different colors

    //assigns blue
    colors[0] = #0FEAF0;

    //assigns pink
    colors[1] = #F00FA5;

    //assigns green
    colors[2] = #59E813;

    //assigns purple
    colors[3] = #AE11ED;

    //assigns red
    colors[4] = #F20F0F;

    //assigns orange
    colors[5] = #F2900F;

    //Instantiates the sun
    //Image of the sun by James Hasting-Trew
    //http://planetpixelemporium.com/venus.html
    //Artist authorizes the use of his image
    //Modified by Maelle Morin
    celestialObjects.add(new Sun(new PVector(0, 0, 0), 450, "sun.jpg"));

    //instantiates the 6 planets
    //Planet Names generated by this website http://fantasynamegenerators.com/planet_names.php#.VxUBQJPhCRu
    //The planet textures were done by David Bryan Roberson(avmorgan) and modified by Maelle Morin
    //All the image are under the Creative Commons Atribution-Share Alike 3.0 License
    //blue : http://fav.me/d5omtsl
    //pink: http://fav.me/d4z4auq
    //green: http://fav.me/d2x26i1
    //purple and orange: http://fav.me/d6aj028
    //red: http://fav.me/d2x26xy
    celestialObjects.add(new Planet(new PVector(0, 0, 0), 60, "blue.jpg", colors[0], 800, 20000, 20, 200, "BLUME"));   
    celestialObjects.add(new Planet(new PVector(0, 0, 0), 100, "pink.jpg", colors[1], 1300, 28000, 20, 200, "FUSTRYRIA"));
    celestialObjects.add(new Planet(new PVector(0, 0, 0), 75, "green.jpg", colors[2], 2000, 45000, 20, 100, "GRERTH"));
    celestialObjects.add(new Planet(new PVector(0, 0, 0), 185, "purple.jpg", colors[3], 3000, 21000, 20, 300, "PUANOPE"));
    celestialObjects.add(new Planet(new PVector(0, 0, 0), 40, "red.jpg", colors[4], 3600, 30000, 20, 100, "RUBLAPUS"));
    celestialObjects.add(new Planet(new PVector(0, 0, 0), 90, "orange.jpg", colors[5], 4000, 60000, 20, 200, "OSMIANUS"));
  }

  //this function displays all the elements in the universe
  void display() {

    //calls the display function of the camera
    myCamera.display();

    //for each celestial object
    for (int i = 0; i <  celestialObjects.size(); i++) {
      CelestialObject co =  celestialObjects.get(i);

      //calls the display and update function
      co.display();
      co.update();
    }
  }

  //Detects collision between the spaceship/camera and the celestial objects
  void collision() {

    //for each celestial objects
    for (int i = 0; i < celestialObjects.size(); i++) {

      CelestialObject co = celestialObjects.get(i);

      //distance between the camera and each celestial object
      PVector distance = PVector.sub(myCamera.getLocation(), co.getLocation());

      //if distance is less than the mass of the celestial object times 4
      if (distance.mag() < co.getMass() * 4) { 

        //sets the distance magnitude to the mass of the celestial object times 4
        distance.setMag(co.getMass() * 4);

        //new camera location by adding the distance to the location of the celestial object
        PVector newCameraLocation = PVector.add(co.getLocation(), distance);

        //sets the new camera location
        myCamera.setLocation(newCameraLocation);
      }
    }
  }

  //returns the background image
  PImage getBackground() {
    return background;
  }
}